####################################################################
# General Parameters
#####################################################################
set opt(title) zero	;
set opt(seed) -1		;
set opt(stop) 100.0	;# Stop time.
set opt(ecn) 0		;
#####################################################################
# Topology
#####################################################################
set opt(rate) 10	;# speed of congested link in Mbps
set opt(delay) 10	;# delay of congested link in ms
set opt(secondDelay) 55	;# average delay of access links in ms
                         # keep this - this is for the range of RTTs. 
set opt(secondRate) 100 ;# speed of access link in Mbps
set opt(thirdDelay) 1   ;# delay for "right-hand" access links.
			 # if 0, delay is 0.
			 # if 1, delay same as for "left-hand" access links.
set opt(rtts) 1		;# 1 for a range of delays for the access links.
			 # 0 for all rtts the same.
			 # 2.E for rtts increasing by epsilon.
set opt(numWebNodes) 10 ;# number of access links for web traffic 
set opt(accessLink) 0   ;# 1 for a range of bandwidth for the access links.
                         # NOT IMPLEMENTED YET
#####################################################################
# AQM parameters
#####################################################################
set opt(minth) 30	;
set opt(maxth) 0	;
set opt(queue) DT       ;# 0 for DropTail
			;# 1 for RED
set opt(adaptive) 1	;# 1 for Adaptive RED, 0 for plain RED 
set opt(qsize) 100	;# Queue size in packets.
set opt(bytes) true	;# true: Queue in byte mode
			 # false: Queue in packet mode
set opt(queue_in_bytes) true ;# RED queue
set opt(DTqueue_in_bytes) true ;# DT queue
set opt(droprate) -1    ;# non-negative for an infinite queue, infinite-
			 # bandwidth link, and fixed packet drop rate.
set opt(dropunit) 1     ;# 1 for per-packet, 2 for per-byte.
#####################################################################
# Traffic generation.
#####################################################################
set opt(flows) 4	;# number of long-lived TCP flows
set opt(tfrc) 1		;# 1 to add tfrc flows; 0 not to add tfrc flows
set opt(voip) 1		;# 1 for old VoIP mode of TFRC
			 # 2 for new, good VoIP mode
                         # 3 for new, good VoIP with ShortIntervals_ 3
                         # 0 not to use VoIP mode
set opt(tfrcsource) 1   ;# 1 for a cbr source, 2 for an infinite-data source
set opt(window) 100	;# window for long-lived traffic
set opt(smallpkt)  -3   ;# inverse of fraction of TCP connections 
                         #    with smaller packets
			 # if -1, all data pkts are 500 bytes.
			 # if -2, all data pkts are 1000 bytes.
			 # if -3, this has no effect on packet size.
set opt(reverse) 1	;# reverse-path traffic
			 # 0 for two flows, -1 for no flows .
set opt(tcptype) 1	;# 1 for SACK, 2 for Tahoe.
set opt(startTime) 1.0	;# start time for flow 1, with time 0 for flow 0
set opt(overhead) 1	;# for random jitter at TCP sender
set opt(pktsize) 1460	;# TCP packet size in bytes
set opt(tfrcpktsize) 200 ;# TFRC packet size in bytes
set opt(tfrcInterval) 100 ;# Interval between packets, for TFRC source.
set opt(cbrrate) 160Kb  ;# CBR rate (200*8*100)
set opt(cbrrate1) 0     ;# CBR rate in bps
set opt(wnd_init_opt) 2 ;# Policy on initial cwnd -- always 2 by default
set opt(cwv) 0		;# Congestion Window Validation (i.e. QOption_)
set opt(timestamps) false ;# true for timestamps in TCP

#####################################################################
# Web traffic
#####################################################################
set opt(web) 10		;# number of web sessions

set opt(numPage) 10     ;# number of pages per session
set opt(pagesize) 10	;# number of objects per page 
set opt(objSize) 10	;# average size of web object in pkts. 
set opt(shape) 1.05	;# shape parameter for Pareto distribution of web size
			 # a larger parameter means more small objects
set opt(interpage) 1	;# interpage parameter for web traffic generator.
set opt(persistent_conns) 0 ;# use persistent web connections
			# not yet implemented
set opt(web_trace) 0    ;# 1 if detailed trace of web events is needed

#####################################################################
# Plotting statistics.
#####################################################################
set opt(intervals) 10	;
set opt(plotQ) 0	;# 1 to plot the instantaneous and average queue 
set opt(printRTTs) 0    ;# 1 to print link delays
set opt(printPkts) 1    ;# 1 to print per-connection stats.
set opt(printFIDs) 1	;# 1 to print per-FID stats
set opt(printDist) 0	;# 1 to print RTT and seqno distributions
set opt(terse) 0        ;# for terse output 
set opt(tfrc-sp) 0      ;# 1 to plot loss interval statistics

set opt(traceall) 0	;# Trace all queue events - Lot's of output!
set opt(reverseMonitor) 0 ;# Monitor also the reverse direction

#####################################################################

remove-all-packet-headers       ; # removes all except common
add-packet-header Flags IP TCP TFRC TFRC_ACK RTP; # hdrs reqd 

proc getopt {argc argv} {
        global opt
        for {set i 0} {$i < $argc} {incr i} {
                set arg [lindex $argv $i]
                if {[string range $arg 0 0] != "-"} continue
                set name [string range $arg 1 end]
                set opt($name) [lindex $argv [expr $i+1]]
		if !$opt(terse) {
                	puts "opt($name): $opt($name)" 
		}
        }
}

getopt $argc $argv
source web.tcl
source plot.tcl

if {$opt(droprate) >= 0.0} {
 	set opt(rate) 100000	
 	set opt(qsize) 10000	
}

## set pkts [expr $opt(pagesize) * $opt(objSize)
## puts "One web session:

if {$opt(seed) > -1} {
        puts "Seeding Random number generator with $opt(seed)\n"
        ns-random $opt(seed)
}
set quarterTime [expr $opt(stop) / 4.0]
set halftime [expr $opt(stop) / 2.0 ]
set endtime [expr $opt(stop) - 2.0 ]

if {$opt(queue) == "DT"} {
  set qm DropTail
}
if {$opt(queue) == "RED"} {
  set qm RED
}
if { $opt(cbrrate1) > 0} {
  set newcbrrate [expr $opt(cbrrate1) * 0.001]
  set opt(cbrrate) [format %s $newcbrrate]Kb
}
set almosttime [expr $opt(stop)- 0.01]
set tcpTick_ 0.01
Agent/TCP set tcpTick_ $tcpTick_
set out $opt(title)

#plot related functions
set quiet "false"
if {$opt(plotQ) > 0} {
  set tchan_ [open "all.q.$out" w]
}

proc stop {} {
	global quiet tchan_ out opt
	if { [info exists tchan_] && $quiet == "false" } {
		plotQueue "$out"
	}
 	exit
}

set ns [new Simulator]

if {$opt(traceall) == 1} {
    set tf [open out-$opt(title).tr w]
    $ns trace-all $tf
}

set node_(r1) [$ns node]
set node_(r2) [$ns node]
set node_(r3) [$ns node]

if {$opt(ecn) == 1} {
   Queue/RED set setbit_ true
}
Queue/RED set summarystats_ true
Queue/DropTail set summarystats_ true
Queue/RED set adaptive_ $opt(adaptive)
Queue/RED set q_weight_ 0.0
Queue/RED set thresh_ $opt(minth)
Queue/RED set maxthresh_ $opt(maxth)
Queue/RED set bytes_ $opt(bytes)
Queue/RED set queue_in_bytes_ $opt(queue_in_bytes) 
Queue/DropTail set queue_in_bytes_ $opt(DTqueue_in_bytes) 

Agent/TCP set ecn_ $opt(ecn) 
Agent/TCP set packetSize_ $opt(pktsize)
Agent/TCP set window_ $opt(window)
Agent/TCP set windowInitOption_ $opt(wnd_init_opt)
Agent/TCP set QOption_ $opt(cwv)
Agent/TCP set timestamps_ $opt(timestamps)

Agent/TFRC set ecn_ $opt(ecn)
Agent/TFRC set packetSize_ $opt(tfrcpktsize)
Agent/TFRC set voip_max_pkt_rate_ $opt(tfrcInterval)
if {$opt(voip) >= 1} {
    Agent/TFRC set voip_ 1
}
if {$opt(voip) == 2} {
    Agent/TFRCSink set ShortIntervals_ 1
    Agent/TFRCSink set ShortRtts_ 2   
}
if {$opt(voip) == 3} {
    Agent/TFRCSink set ShortIntervals_ 3
    Agent/TFRCSink set ShortRtts_ 3   
}
if {$opt(tfrc-sp) == 1} {
    Agent/TFRCSink set printLosses_ 1
    Agent/TFRCSink set printLoss_ 1
}

if {$opt(overhead) > 0} {
  Agent/TCP set overhead_ [expr 8*$opt(pktsize)/($opt(rate)*1000000) ]
}

QueueMonitor set keepRTTstats_ 1
QueueMonitor set maxRTT_ 1
QueueMonitor set binsPerSec_ 100
QueueMonitor set keepSeqnoStats_ 1
QueueMonitor set maxSeqno_ 2000
#QueueMonitor set SeqnoBinSize_ 100
QueueMonitor set SeqnoBinSize_ 1
#QueueMonitor set SeqnoBinSize_ 10

$ns duplex-link $node_(r1) $node_(r2) [set opt(rate)]Mb [expr $opt(delay)/2]ms $qm
$ns duplex-link $node_(r2) $node_(r3) [set opt(rate)]Mb [expr $opt(delay)/2]ms $qm
$ns queue-limit $node_(r1) $node_(r2) $opt(qsize)
$ns queue-limit $node_(r2) $node_(r1) $opt(qsize)
$ns queue-limit $node_(r2) $node_(r3) $opt(qsize)
$ns queue-limit $node_(r3) $node_(r2) $opt(qsize)

proc setLossyLink { droprate } {
    global lossylink_ ns node_ opt
    set lossylink_ [$ns link $node_(r1) $node_(r2)]
    set em [new ErrorModule Fid]
    if {$opt(dropunit) == 1} {set drunit pkt}
    if {$opt(dropunit) == 2} {set drunit byte}
    set errmodel [new ErrorModel/Uniform $droprate $drunit ]
    if {$opt(ecn) == 1} {
        $errmodel set markecn_ true
    }
    $lossylink_ errormodule $em
    $em insert $errmodel
    $em bind $errmodel 0 5  
    $em default pass
}

# proc setLoss {} {
#     global lossmodel ns opt lossylink_
#     set errmodule [$lossylink_ errormodule]
#     set lossmodel [$errmodule errormodels]
#     $lossmodel set offset_ $opt(droprate) 
#     $lossmodel set burstlen_ 1
#     $lossmodel set period_ $opt(droprate)
# }

if {$opt(droprate) >= 0} {
    setLossyLink $opt(droprate)
}

set slink [$ns link $node_(r1) $node_(r2)] ; 
set rlink [$ns link $node_(r2) $node_(r1)] ; 
set fmon [$ns makeflowmon Fid]
$ns attach-fmon $slink $fmon

# PS
if {$opt(reverseMonitor)} {
    $ns attach-fmon $rlink $fmon
}

set squeue [$slink queue]
set rqueue [$rlink queue]

if {$opt(printDist) == 1 } {
  set fmon1 [new QueueMonitor]
  # These two lines are new:
  set outfile [open $opt(title).stats w]
  $fmon1 traceDist $outfile
  #
  $ns attach-fmon $slink $fmon1
}

# trace related
if {$opt(plotQ) > 0} {
  $squeue attach $tchan_
  $squeue trace curq_
  $squeue trace ave_
  $squeue trace cur_max_p_
}

# Set up forward TCP and TFRC connections, FIDs 0 (TCP) and 5 (TFRC)
for {set i 0} {$i < $opt(flows)} {incr i} {
    set node_(s$i) [$ns node]
    set node_(k$i) [$ns node]
    if {$opt(rtts) == 0 } {
	set delay2 $opt(secondDelay)
    } elseif {$opt(rtts) >= 2 } {
	set extra [expr 250 * $i * ($opt(rtts)-2.0)]
	# Times 1000 for ms instead of seconds.  Divide by 4 for 2 links.
	set delay2 [expr $opt(secondDelay) + $extra]
        puts "flow $i delay $delay2"
    } else {
	## We are not randomizing the link delays.
	## For long-lived traffic, the shorter RTTs dominate.
        set num [expr (($i)%10)/9.0]
	## num is a number between 0 and 1
	set delay2 [expr 2*$opt(secondDelay)*$num]
    }
    if {$opt(thirdDelay) == 0} {
	set delay2a 0.0
    } else {
        set delay2a $delay2
    }
    if {$opt(printRTTs) == 1} {
      puts "link delay forw [format "%.2f" $delay2] [format "%.2f" $delay2a]"
    }
    set rate $opt(secondRate)
    $ns duplex-link $node_(s$i) $node_(r1) [expr $rate]Mb [expr $delay2]ms DropTail
    $ns duplex-link $node_(k$i) $node_(r3) [expr $rate]Mb [expr $delay2a]ms DropTail
    if {$opt(tcptype) == 1} {
      set tcp$i [$ns create-connection TCP/Sack1 $node_(s$i) TCPSink/Sack1 $node_(k$i) 0]
    } elseif {$opt(tcptype) == 0} {
      set tcp$i [$ns create-connection TCP $node_(s$i) TCPSink $node_(k$i) 0]
    } 
    if {$opt(smallpkt)==-1 || ($opt(smallpkt)>0 && $i%$opt(smallpkt)==0)} {
      [set tcp$i] set packetSize_ 500
    }
    set ftp$i [[set tcp$i] attach-app FTP]
    set tmp [new RandomVariable/Uniform]
    set starttime [expr 1.0 * [$tmp value] * $quarterTime ]
    $ns at $starttime "[set ftp$i] start"
    $ns at $endtime "[set ftp$i] stop"

    if {$opt(tfrc) == 1} {
      set tfrc$i [$ns create-connection TFRC $node_(s$i) TFRCSink $node_(k$i) 5]
      if {$opt(tfrcsource) == 1} {
  	[set tfrc$i] set SndrType_ 1
  	set cbr [new Application/Traffic/CBR]
  	$cbr attach-agent [set tfrc$i]
  	$cbr set rate_ $opt(cbrrate)
  	$cbr set packetSize_ $opt(tfrcpktsize)
          set starttime1 [expr 1.0 * [$tmp value] * $quarterTime + 0.001]
          $ns at $starttime1 "$cbr start"
          $ns at $endtime "$cbr stop"
      }
      if {$opt(tfrcsource) == 2} {
  	Agent/TFRC set packetSize_ $opt(tfrcpktsize)
  	$ns at $starttime "[set tfrc$i] start"
  	$ns at $opt(stop) "[set tfrc$i] stop"
      }
      if {$opt(printPkts) == 1} {
          $ns at $halftime "printpkts $i [set tcp$i] [set tfrc$i]"
          $ns at $almosttime "printpkts $i [set tcp$i] [set tfrc$i]"
      }
    } else {
      if {$opt(printPkts) == 1} {
          $ns at $halftime "printpkts $i [set tcp$i] -1"
          $ns at $almosttime "printpkts $i [set tcp$i] -1"
      }
    }
}
#####set cbr [$tfrc attach-app Traffic/CBR]
#$cbr set packetSize_ $opt(tfrcpktsize)
#100 200-byte pkts per second
#rate 160Kb
#	$cbr set rate_ 160Kb


# Traffic on the reverse path.  FID 1.
# Reverse-path traffic is half the number of flows as for the forward path.
set randomflows [expr $opt(flows) / 2 ]
if {$randomflows < 2} { set randomflows 2 }
if {$opt(reverse)==0} { set randomflows 2 } 
if {$opt(reverse)==-1} { set randomflows 0 } 
if {$opt(reverse)==2} {
    Agent/TCP set window_ 8
}
for {set i 0} {$i < $randomflows} {incr i} {
    set node_(sR$i) [$ns node]
    set node_(kR$i) [$ns node]
    if {$opt(rtts) == 0 } {
	set delay2 $opt(secondDelay)
	set delay2a $opt(secondDelay)
    } else {
	## We are not randomizing the link delays.
	## For long-lived traffic, the shorter RTTs dominate.
        set num [expr (($i+4)%10)/9.0]
	## num is a number between 0 and 1
	set delay2 [expr 2*$opt(secondDelay)*$num]
        set delay2a [expr 2*$opt(secondDelay)*$num]
    }
    if {$opt(printRTTs) == 1} {
      puts "link delay rev [format "%.2f" $delay2] [format "%.2f" $delay2a]"
    }
    $ns duplex-link $node_(sR$i) $node_(r1) 100Mb [expr $delay2]ms DropTail
    $ns duplex-link $node_(kR$i) $node_(r3) 100Mb [expr $delay2a]ms DropTail
    set tcp$i [$ns create-connection TCP/Sack1 $node_(kR$i) TCPSink/Sack1 $node_(sR$i) 1]
    if {$i % $opt(smallpkt) == 0} {
      [set tcp$i] set packetSize_ 500
    }

    set ftp$i [[set tcp$i] attach-app FTP]
    set tmp [new RandomVariable/Uniform]
    set starttime [expr 1.0 * [$tmp value] * $quarterTime ]
    if {$opt(startTime)>0} {
      if {$i == 0} {
        set starttime 0.0
      } elseif {$i == 1} {
        set starttime $opt(startTime)
      }
    }
    $ns at $starttime "[set ftp$i] start"
    $ns at $opt(stop) "[set ftp$i] stop"
}

Agent/TCP set window_ 64
set count 100  
#(FID for forward web traffic?)
# Add forward web traffic.  FID 2.
add_web_traffic $opt(secondDelay) $opt(web) $opt(interpage) $opt(pagesize) $opt(objSize) $opt(shape) 1 400 10 0
## paper: add_web_traffic $opt(secondDelay) $opt(web) 1 10 $opt(objSize)
##        add_web_traffic {bdel nums interpage pagesize objectsize}
# Add reverse web traffic. FID 3.
add_web_traffic $opt(secondDelay) [expr $opt(web)/2] $opt(interpage) $opt(pagesize) $opt(objSize) $opt(shape) 0 10000 10 0

for {set i $opt(intervals)} {$i < $opt(stop) + $opt(intervals)} {incr i $opt(intervals)} {
  # $ns at $i.0 "$squeue printstats"
  # $ns at $i.0 "printall $fmon $opt(stop) $opt(rate) $i"
}

if {$opt(printDist) == 1 } {
  $ns at [expr $opt(stop) - 0.003] "$fmon1 printRTTs"
  $ns at $halftime "$fmon1 printSeqnos"
}
if {$opt(printFIDs) == 1 } {
  $ns at $halftime "printtime; printdrops 0 $fmon; printdrops 5 $fmon;" 
  $ns at [expr $opt(stop) - 0.004]  "printtime; printdrops 0 $fmon; printdrops 5 $fmon;"
}
if {$opt(printDist) == 1 } {
  $ns at $opt(stop) "$fmon1 printSeqnos"
}
if {$opt(plotQ) == 1} {
  set link1 [$ns link $node_(r1) $node_(r2)]
  set queue1 [$link1 queue]
  $ns at [expr $opt(stop) - 0.002]  "$queue1 printstats"
}
if $opt(terse) {
  $ns at 0.0 {puts -nonewline "$opt(flows)"}
  $ns at $opt(stop) {puts " "}
} else {
  $ns at 0.0 {puts "flows: $opt(flows)"}
}
#$ns at $halftime "printall $fmon $opt(stop) $opt(rate) $halftime"
#$ns at $opt(stop) "printall $fmon $opt(stop) $opt(rate) $opt(stop)"
$ns at $opt(stop) "stop"

$ns run

